# -*- coding: utf-8 -*-
import random


__all__ = ['Name']


class name(object):
    def is_male(self):
        from gimei import MALE
        return self.gender == MALE

    def is_female(self):
        from gimei import FEMALE
        return self.gender == FEMALE

    @property
    def kanji(self):
        return self.all[0]

    @property
    def hiragana(self):
        return self.all[1]

    @property
    def katakana(self):
        return self.all[2]

    @classmethod
    def get_all_names(cls):
        from gimei import Gimei
        return Gimei.NAMES[cls.__name__]


class first_name(name):
    def __init__(self, gender=None):
        from gimei import GENDER
        from gimei import Gimei
        self.gender = gender if gender is not None else random.choice(GENDER)
        self.all = random.choice(Gimei.NAMES[self.__class__.__name__][self.gender])


class last_name(name):
    def __init__(self, gender=None):
        from gimei import GENDER
        from gimei import Gimei
        self.gender = gender if gender is not None else random.choice(GENDER)
        self.all = random.choice(Gimei.NAMES[self.__class__.__name__])


class Name(object):
    def __init__(self, gender=None):
        if gender is None:
            from gimei import GENDER
            gender = random.choice(GENDER)
        self.gender = gender
        self.first = first_name(self.gender)
        self.last = last_name(self.gender)

    @property
    def kanji(self):
        return u"{} {}".format(
            self.last.kanji,
            self.first.kanji,
        )

    @property
    def hiragana(self):
        return u"{} {}".format(
            self.last.hiragana,
            self.first.hiragana,
        )

    @property
    def katakana(self):
        return u"{} {}".format(
            self.last.katakana,
            self.first.katakana,
        )

    @property
    def is_male(self):
        from gimei import MALE
        return self.gender == MALE

    @property
    def is_female(self):
        from gimei import FEMALE
        return self.gender == FEMALE

    def __repr__(self):
        return self.kanji.encode('utf-8')

    @classmethod
    def find_name_by_index(cls, name, idx):
        from gimei import MALE, FEMALE
        result = []

        token = name.split(' ')
        if len(token) != 2:
            return None
        first_names = first_name.get_all_names()
        last_names = last_name.get_all_names()

        for _names in last_names:
            if _names[idx] == token[0]:
                result.append(_names[idx])
                break

        for _names in first_names[MALE]:
            if _names[idx] == token[1]:
                result.append(token[1])
                break

        if len(result) != 2:
            for _names in first_names[FEMALE]:
                if _names[idx] == token[1]:
                    result.append(token[1])
                    break

        if len(result) == 2:
            return result
        return None

    @classmethod
    def find_name_by_kanji(cls, kanji):
        return cls.find_name_by_index(kanji, 0)

    @classmethod
    def find_name_by_hiragana(cls, hiragana):
        return cls.find_name_by_index(hiragana, 1)

    @classmethod
    def find_name_by_katakana(cls, katakana):
        return cls.find_name_by_index(katakana, 2)



# coding: utf-8
from gimei import Gimei
from gimei import Name


class TestName(object):
    def test_kanji_name_in_data(self):
        name = Gimei().name
        assert Name.find_name_by_kanji(name.kanji)

    def test_hiragana_name_in_data(self):
        name = Gimei().name
        assert Name.find_name_by_hiragana(name.hiragana)

    def test_katakana_name_in_data(self):
        name = Gimei().name
        assert Name.find_name_by_katakana(name.katakana)

    def test_create_name_from_gender(self):
        name = Gimei('male').name
        assert name.is_male

    def test_create_name_from_gender(self):
        name = Gimei('female').name
        assert name.is_female



# -*- coding: utf-8 -*-
import random
import yaml
try:
    from yaml import CLoader as Loader
except ImportError:
    from yaml import Loader
from address import Address
from name import Name


MALE = 'male'
FEMALE = 'female'
GENDER = (MALE, FEMALE)


def yaml_load(file_path, mode='r'):
    import os
    file_dir = os.path.abspath(os.path.join(os.path.dirname(__file__)))
    try:
        f = open(file_dir + '/' + file_path, mode)
    except IOError:
        raise Exception("Can not open {}".format(file_dir + '/' + file_path))
    return yaml.load(f, Loader=Loader)


class Gimei(object):
    NAMES = yaml_load('./data/names.yml')
    ADDRESSES = yaml_load('./data/addresses.yml')

    def __init__(self, gender=None):
        if gender is None:
            gender = random.choice(GENDER)
        self.gender = gender

    @property
    def name(self):
        return Name(self.gender)

    @property
    def address(self):
        return Address()



# -*- coding: utf-8 -*-

import random


__all__ = ['Address']


class Prefecture(object):
    def __init__(self):
        from gimei import Gimei
        self.prefectures = random.choice(Gimei.ADDRESSES['addresses']['prefecture'])

    @property
    def kanji(self):
        return u"{}".format(
            self.prefectures[0]
        )

    @property
    def hiragana(self):
        return u"{}".format(
            self.prefectures[1]
        )

    @property
    def katakana(self):
        return u"{}".format(
            self.prefectures[2]
        )


class City(object):
    def __init__(self):
        from gimei import Gimei
        self.cities = random.choice(Gimei.ADDRESSES['addresses']['city'])

    @property
    def kanji(self):
        return u"{}".format(
            self.cities[0]
        )

    @property
    def hiragana(self):
        return u"{}".format(
            self.cities[1]
        )

    @property
    def katakana(self):
        return u"{}".format(
            self.cities[2]
        )


class Town(object):
    def __init__(self):
        from gimei import Gimei
        self.towns = random.choice(Gimei.ADDRESSES['addresses']['town'])

    @property
    def kanji(self):
        return u"{}".format(
            self.towns[0]
        )

    @property
    def hiragana(self):
        return u"{}".format(
            self.towns[1]
        )

    @property
    def katakana(self):
        return u"{}".format(
            self.towns[2]
        )


class Address(object):
    def __init__(self):
        self.prefecture = Prefecture()
        self.city = City()
        self.town = Town()

    @property
    def kanji(self):
        return u"{}{}{}".format(
            self.prefecture.kanji,
            self.city.kanji,
            self.town.kanji,
        )

    @property
    def hiragana(self):
        return u"{}{}{}".format(
            self.prefecture.hiragana,
            self.city.hiragana,
            self.town.hiragana,
        )

    @property
    def katakana(self):
        return u"{}{}{}".format(
            self.prefecture.katakana,
            self.city.katakana,
            self.town.katakana,
        )

    def __repr__(self):
        return self.kanji.encode('utf-8')

    @classmethod
    def get_prefectures(cls):
        from gimei import Gimei
        return Gimei.ADDRESSES['addresses']['prefecture']

    @classmethod
    def get_cities(cls):
        from gimei import Gimei
        return Gimei.ADDRESSES['addresses']['city']

    @classmethod
    def get_towns(cls):
        from gimei import Gimei
        return Gimei.ADDRESSES['addresses']['town']

    @classmethod
    def find_address_by_index(cls, a, idx):
        if not a:
            return None

        for prefecture in cls.get_prefectures():
            if not a.startswith(prefecture[idx]):
                continue
            for city in cls.get_cities():
                if not a.startswith(prefecture[idx] + city[idx]):
                    continue
                for town in cls.get_towns():
                    if not a.startswith(prefecture[idx] + city[idx] + town[idx]):
                        continue
                    return prefecture[idx] + city[idx] + town[idx]
        return None

    @classmethod
    def find_address_by_kanji(cls, kanji):
        return cls.find_address_by_index(kanji, 0)

    @classmethod
    def find_address_by_hiragana(cls, hiragana):
        return cls.find_address_by_index(hiragana, 1)

    @classmethod
    def find_address_by_katakana(cls, katakana):
        return cls.find_address_by_index(katakana, 2)



# coding: utf-8
from gimei import Gimei
from gimei import Address

class TestAddress(object):
    def test_kanji_address_in_data(self):
        address = Gimei().address
        assert Address.find_address_by_kanji(address.kanji)

    def test_hiragana_address_in_data(self):
        address = Gimei().address
        assert Address.find_address_by_hiragana(address.hiragana)

    def test_katakana_address_in_data(self):
        address = Gimei().address
        assert Address.find_address_by_katakana(address.katakana)



from setuptools import setup, find_packages


with open("README.rst",  "rt") as f: readme = f.read()


setup(
    name='gimei',
    version="0.1.53",
    description="generates the name and the address at random.",
    long_description=__doc__,
    author='Mao Nabeta',
    author_email='mao.nabeta@gmail.com',
    url='https://github.com/nabetama/gimei',
    packages=find_packages(),
    include_package_data=True,
    install_requires=['pyyaml'],
    provides=['gimei', 'name', 'random'],
    keywords=['gimei', 'name', 'random'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: Communications :: Chat',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ]
    )
